SystemOrganization addCategory: #'Amber-Tests-Slime'!

!SequenceableCollection methodsFor: '*amber-tests-slime' stamp: 'PaulDeBruicker 1/12/2012 15:23'!
endsWithSubCollection: aSequenceableCollection
	"Some platforms implement #endsWith: to answer true for an empty argument."

	^ self endsWith: aSequenceableCollection! !

TestCase subclass: #AMSlimeTest
	instanceVariableNames: 'model environment'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Amber-Tests-Slime'!

!AMSlimeTest commentStamp: 'PaulDeBruicker 1/12/2012 09:41' prior: 0!
The structure of this is just copied from Lukas Renggli's implementation for Grease.  !

!AMSlimeTest methodsFor: 'accessing-code' stamp: 'PaulDeBruicker 1/12/2012 09:31'!
canParse: aString
	RBParser
		parseMethod: aString
		onError: [ :err :pos | ^ false ].
	^ true! !

!AMSlimeTest methodsFor: 'accessing' stamp: 'PaulDeBruicker 1/12/2012 09:31'!
category
	^ self class category , '-Data'! !

!AMSlimeTest methodsFor: 'accessing-code' stamp: 'PaulDeBruicker 1/12/2012 09:31'!
compile: aString in: aClass
	^ aClass compile: aString classified: #(accessing)! !

!AMSlimeTest methodsFor: 'accessing-code' stamp: 'PaulDeBruicker 1/12/2012 09:31'!
defineClass: aClassSymbol superclass: aSuperSymbol
	| class |
	model defineClass: aSuperSymbol , ' subclass: #' , aClassSymbol , '
		instanceVariableNames: '''' 
		classVariableNames: ''''
		poolDictionaries: ''''
		category: ''' , self category , ''''.
	class := model classNamed: aClassSymbol.
	environment 
		addClass: class;
		addClass: class theMetaClass.
	^ class! !

!AMSlimeTest methodsFor: 'accessing' stamp: 'PaulDeBruicker 1/12/2012 09:32'!
rules
	^ OrderedCollection new
		addAll: AMSlimeBlockLintRule allSubclasses;
	"	addAll: GRSlimeParseTreeLintRule allSubclasses;"
		addAll: AMSlimeTransformationRule allSubclasses;
		yourself! !

!AMSlimeTest methodsFor: 'running-checks' stamp: 'PaulDeBruicker 1/12/2012 14:06'!
run: aClass do: aBlock
	| rules rule composite undo |
	rules := self rules
		collect: [ :each | each new ].
	rule := rules
		detect: [ :each | each isKindOf: aClass ]
		ifNone: [ self error: aClass name , ' is not a slime rule' ].
	self
		assert: (rule name isString and: [ rule name notEmpty ])
		description: aClass name , ' has no name'.
	self
		assert: (rule rationale isString and: [ rule rationale notEmpty and: [ rule rationale endsWithSubCollection: '.' ] ])
		description: aClass name , ' has no rationale'.
	composite := RBCompositeLintRule rules: rules name: 'Slime'.
	undo := model changes execute.
	[	RBSmalllintChecker runRule: composite onEnvironment: environment.
		rules do: [ :each |
			self 
				assert: (rule = each or: [ each result isEmpty or: [ each changes isEmpty ] ])
				description: each name , ' should not report errors' ].
		aBlock value: rule value: rules ]
			ensure: [ undo execute ].
	^ rule! !

!AMSlimeTest methodsFor: 'running' stamp: 'PaulDeBruicker 1/12/2012 09:31'!
runCase
	SystemChangeNotifier uniqueInstance
		doSilently: [ super runCase ]! !

!AMSlimeTest methodsFor: 'running-checks' stamp: 'PaulDeBruicker 1/12/2012 09:31'!
runRule: aClass selectors: aCollection
	self run: aClass do: [ :rule :rules |
		rule result classesAndSelectorsDo: [ :class :selector |
			self
				assert: (aCollection includes: selector)
				description: selector printString , ' should not be reported' ].
		aCollection do: [ :selector |
			self
				assert: (rule result classes
					anySatisfy: [ :class | rule result includesSelector: selector in: class ])
				description: selector printString , ' should be reported' ] ]! !

!AMSlimeTest methodsFor: 'running-checks' stamp: 'PaulDeBruicker 1/12/2012 09:31'!
runTransformation: aClass changes: aCollection
	| expected |
	expected := aCollection 
		collect: [ :each | RBParser parseMethod: each ].
	self run: aClass do: [ :rule :rules |
		rule changes do: [ :change |
			self
				assert: (expected anySatisfy: [ :parseTree | change parseTree = parseTree ])
				description: change selector , ' should not be transformed' ].
		expected do: [ :parseTree |
			self
				assert: (rule changes anySatisfy: [ :change | change parseTree = parseTree ])
				description: parseTree selector , ' should be transformed' ] ]! !

!AMSlimeTest methodsFor: 'running' stamp: 'PaulDeBruicker 1/12/2012 09:31'!
setUp
	super setUp.
	model := RBNamespace new.
	environment := RBClassEnvironment new! !

!AMSlimeTest methodsFor: 'running' stamp: 'PaulDeBruicker 1/12/2012 09:31'!
tearDown
	super tearDown.
	Smalltalk organization
		removeCategory: self category! !

!AMSlimeTest methodsFor: 'tests-transform' stamp: 'PaulDeBruicker 1/12/2012 09:42'!
testCharacterLiteralToString
	| class |
	class := self
		defineClass: #AMSlimeMockObject
		superclass: #Object.
	self compile: 'one $a' in: class.
		self compile: 'two $a. $b' in: class.
	
	self 
		runTransformation: AMCharacterLiteralToStringRule
		changes: #(
			'one ''a''.'
			'two ''a''. ''b''' )! !

!AMSlimeTest methodsFor: 'tests-transform' stamp: 'PaulDeBruicker 1/12/2012 11:27'!
testDoesNotSetClassCategory
	| class |
	class := self defineClass: #AMSlimeMockObject superclass: #Object.
	self compile: 'invalid SystemOrganizer addCategory: ''Amber-Mock-Category''' in: class.
	self runRule: AMDontCreateCategoriesInAmberRule selectors: #(#invalid)! !

!AMSlimeTest methodsFor: 'tests-transform' stamp: 'PaulDeBruicker 1/12/2012 10:48'!
testNoClassVariables
	| class |
	class := self defineClass: #AMSlimeMockObject superclass: #Object.
	class addClassVariable: 'AmberClassVar'.
	self runRule: AMNoClassVariablesInAmberRule selectors: #()! !

!AMSlimeTest methodsFor: 'tests-transform' stamp: 'PaulDeBruicker 1/12/2012 10:48'!
testNoPragmas
	| class |
	class := self defineClass: #AMSlimeMockObject superclass: #Object.
	self compile: 'invalid <primitive: 137>' in: class.
	self runRule: AMNoPragmasInAmberRule selectors: #(#invalid)! !

!AMSlimeTest methodsFor: 'tests-transform' stamp: 'PaulDeBruicker 1/12/2012 10:48'!
testNoSharedPools
	| class |
	class := self defineClass: #AMSlimeMockObject superclass: #Object.
	class addPoolDictionary: 'ChronologyConstants'.
	self runRule: AMNoSharedPoolsInAmberRule selectors: #()! !

!AMSlimeTest methodsFor: 'tests-transform' stamp: 'PaulDeBruicker 1/12/2012 14:04'!
testSmalltalkGlobalWarning
	| class |
	class := self defineClass: #AMSlimeMockObject superclass: #Object.
	self compile: 'validWarning Smalltalk current at: #Time' in: class.
	self runRule: AMSmalltalkGlobalRule selectors: #(#validWarning)! !

!AMSlimeTest methodsFor: 'tests-transform' stamp: 'PaulDeBruicker 1/12/2012 13:02'!
testSmalltalkToSmalltalkCurrent
	| class |
	class := self defineClass: #AMSlimeMockObject superclass: #Object.
	self compile: 'valid Smalltalk current at: #Time' in: class.
	self compile: 'invalid Smalltalk at: #Time' in: class.
	self runTransformation: AMSmalltalkToSmalltalkCurrentRule changes: #('invalid Smalltalk current at: #Time')! !

Object subclass: #AmberMockObject
	instanceVariableNames: ''
	classVariableNames: 'AmberMockClassVar'
	poolDictionaries: 'ChronologyConstants'
	category: 'Amber-Tests-Slime'!

!AmberMockObject methodsFor: 'as yet unclassified' stamp: 'PaulDeBruicker 1/12/2012 10:11'!
examplePragma
	"Time primSecondsClock"
	<primitive: 137>! !

!AmberMockObject methodsFor: 'as yet unclassified' stamp: 'PaulDeBruicker 1/12/2012 12:54'!
invalidSmalltalkReference
	Smalltalk at: #Time! !

!AmberMockObject methodsFor: 'as yet unclassified' stamp: 'PaulDeBruicker 1/12/2012 10:05'!
oneCharacter
	$a! !

!AmberMockObject methodsFor: 'as yet unclassified' stamp: 'PaulDeBruicker 1/12/2012 11:40'!
setClassCategory
	SystemOrganizer new addCategory: 'Amber-Tests-Slime'! !

!AmberMockObject methodsFor: 'as yet unclassified' stamp: 'PaulDeBruicker 1/12/2012 10:05'!
twoCharacters
	$a.
	$b! !

!AmberMockObject methodsFor: 'as yet unclassified' stamp: 'PaulDeBruicker 1/12/2012 12:54'!
validSmalltalkReference
	Smalltalk current at: #Time! !
